@@ -730,7 +730,7 @@ setMethod("genericGetExpected", signature("MxExpectationLISREL"),
 			  ret[['thresholds']] <- thr
 		  }
 		  zcomp <- sapply(ret, function(x){prod(dim(x))}) == 0
-		  zcomp <- zcomp[!(names(zcomp) %in% 'thresholds')] # Don't care about missing thresholds
+		  zcomp <- zcomp[!(names(zcomp) %in% c('thresholds', 'means'))]
 		  if(any(zcomp)){
 		    stop(paste('Requested component(s)', omxQuotes(names(zcomp)[zcomp]), "had dimension or length 0.\nRequested component not in model. All I have is yours, but I ain't got no", omxQuotes(names(zcomp)[zcomp])))
 		  }
@@ -743,8 +743,10 @@ setMethod("genericGetExpected", signature("MxExpectationLISREL"),
 
 #------------------------------------------------------------------------------
 setMethod("genericGenerateData", signature("MxExpectationLISREL"),
-	function(.Object, model, nrows, subname) {
-		return(generateNormalData(model, nrows, subname))
+	function(.Object, model, nrows, subname, empirical, returnModel, use.miss,
+		   .backend, nrowsProportion) {
+		return(generateNormalData(model, nrows, subname, empirical, returnModel, use.miss,
+		   .backend, nrowsProportion))
 })
 
 
