@@ -446,7 +446,7 @@ setReplaceMethod("[[", "MxMatrix", function(x,i,value) { .mxMatSetLayer(x,i,valu
 
 setMethod("names", "MxMatrix", slotNames)
 
-matrixTypes <- c("Diag", "Full", "Iden", "Lower", "Stand", "Sdiag", "Symm", "Unit", "Zero")
+matrixTypes <- c("Full", "Diag", "Iden", "Lower", "Sdiag", "Stand", "Symm", "Unit", "Zero")
 squareMatrices <- c("Diag", "Iden", "Lower", "Stand", "Sdiag", "Symm")
 fixedMatrices <- c("Iden","Unit","Zero")
 
@@ -494,7 +494,7 @@ mxMatrix <- function(type = c("Full", 'Diag', 'Iden', 'Lower',
 	if (length(list(...)) > 0) {
 		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
 	}
-	type <- match.barg(type)
+	type <- match.barg(as.character(type), choices=matrixTypes)
 	if (missing(dimnames) && !missing(values) && !is.null(dimnames(values))) {
 		dimnames <- dimnames(values)
 	}
