@@ -270,6 +270,6 @@ STARTSM <- mxModel("STARTS",
 startsModel <- mxRun(STARTSM)
 omxCheckCloseEnough(startsModel$output$fit, 2718.410, .1)
 
-if (.Platform$OS.type != 'windows' && detectCores() > 1) {
+if (.Platform$OS.type != 'windows' && parallel::detectCores() > 1) {
 	omxCheckTrue(startsModel$compute$steps[['GD']]$output$maxThreads > 1)
 }
